%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data derived from a NIST high performance ceramics test
% Characterize effect of grinding parameters on sintered,
% reaction-bonded silicone nitride
%
% Solutions developed by Jun-Goo Kwak
% 2-10-19 
% Version 4
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Determine effects of machining factors on cermaic strength

% Number of observations: 32 (full 2^5 factorial design)
% Variable 1 = Table Speed - (slow (.025 m/s) and fast (.125 m/s)) 
% Variable 2 = Feed Rate - (slow (.05 mm) and fast (.125 mm)) 
% Variable 3 = Wheel Grit - (140/170 and 80/100) 
% Variable 4 = Direction - (longitudinal and transverse) 
% Variable 5 = Batch - (1 and 2)

clear;
close all;
clc;

%% Problem 1 - Sorting response data

[factors, strength, run_Order] = data();
response = [strength run_Order];
response_By_Run = sortrows(response,2);

%% Problem 1 

% Plot the response data and see if it comes from normal distribution 
figure('Name','Figure 1');
subplot(2,2,1);
h = normplot(response_By_Run(1:32));
xlabel('Theoretical Quantities');
ylabel('Sample Quantities');

% Box plot to see data distribution 
subplot(2,2,2);
h2 = boxplot(response_By_Run(1:32));
ylabel('Strength')

% Histogram to see data distribution, add additional bins
subplot(2,2,3);
h3 = histogram(response_By_Run(1:32),7);
xlabel('Strength');
ylabel('Frequency');

% Plot response vs run to see if there are time effects 
subplot(2,2,4);
h4 = plot(response_By_Run(:,2), response_By_Run(:,1));
xlabel('Actual Run Order');
ylabel('Strength'); 

% Measures of central tendency
mean_Ceramic_Strength = mean(strength);
std_Ceramic_Strength  = std(strength);
var_Ceramic_Strength  = var(strength);

% ttest - Does not reject the null hypothesis at 5% significance
% vartest - Does not reject the null hypothesis at 5% significance
% Data comes from normal distribution, as evidenced by normfit values 
[h, p, ci] = ttest(strength, mean_Ceramic_Strength, 'Alpha', 0.05);
[h, p, ci]= vartest(strength, var_Ceramic_Strength, 'Alpha',  0.05);
[muHat,sigmaHat] = normfit(strength);

pause(1) % Pause MATLAB execution, otherwise transfers fig 2 to fig 1

%% Problem 2 
% Assess visually, effects of factors on response using boxplot

% Factor 1: Table Speed
figure('Name','Figure 2');
subplot(2,3,1);
a = sortrows([factors(1:32, 1), response(1:32, 1)],1);
a1 = boxplot(a(:,2),a(:,1));
xlabel('Table Speed');
ylabel('Strength');

% Factor 2: Feed Rate
subplot(2,3,2)
b = sortrows([factors(1:32, 2), response(1:32, 1)],1);
b1 = boxplot(b(:,2),b(:,1));
xlabel('Feed Rate');
ylabel('Strength');

% Factor 3: Wheel Grit
subplot(2,3,3)
c = sortrows([factors(1:32, 3), response(1:32, 1)],1);
c1 = boxplot(c(:,2),c(:,1));
xlabel('Wheel Grit');
ylabel('Strength');

% Factor 4: Direction
subplot(2,3,4)
d = sortrows([factors(1:32, 4), response(1:32, 1)],1);
d1 = boxplot(d(:,2),d(:,1));
xlabel('Direction');
ylabel('Strength');

% Factor 5: Batch
subplot(2,3,5)
e = sortrows([factors(1:32, 5), response(1:32, 1)],1);
e1 = boxplot(e(:,2),e(:,1));
xlabel('Batch');
ylabel('Strength');

%% Create the theoretical model and fit model to data
% Remove 4th and 5th order terms, and do 26 order ANVOA.

varnames = {'Table Speed', 'Feed Rate',	'Wheel Grit', ...
    'Direction', 'Batch'};
[anova,tbl] = anovan(strength,factors,3,3,varnames);

% Determine which ANOVA coefficients to use based on p-value 
tmp = cell2mat(tbl(2:26,7));

% Select p-Values < 0.05
coeff = [tmp(1),tmp(2),tmp(3),tmp(4),tmp(5),tmp(6),tmp(7),tmp(8), ...
         tmp(11),tmp(13),tmp(17)];

% Get corresponding strength values based on selected p-value variables
test2 = [strength(1); strength(2); strength(3); strength(4); ...
         strength(5); strength(6); strength(7); strength(8); ...
         strength(11); strength(13); strength(15); strength(17)];

% Get factors from design matrix based on selected strength values
newFactors = [factors(1,:); factors(2,:); factors(3,:); ...
              factors(4,:); factors(5,:); factors(6,:); ...
              factors(7,:); factors(8,:); factors(11,:);...
              factors(13,:); factors(15,:); factors(17,:)];

%% Problem 3 
% Create nonlinear model fit based on interactions

beta0 = [1; 1; 1; 1; 1; 1; 1; 1; 1; 1; 1; 1; 1];
beta_Values = nlinfit(factors,strength,@modelFun,beta0)

% Derive beta coefficients and then use in nlintool
nlintool(factors,strength,@modelFun,beta_Values,0.01);

